import { parseString } from "xml2js";
import fs from "fs";

const arrayToCsv = (data) => {
  return data
    .map((row) =>
      row
        .map(String)
        .map((v) => v.replaceAll('"', '""'))
        .map((v) => `"${v}"`)
        .join(",")
    )
    .join("\r\n");
};

const filterByFileAndTestCase = (data) => {
  return data.map((item, index) => {
    const fileTestCaseData = [];
    if (item.testcase) {
      item.testcase.map((testCase, testIndex) => {
        const testcaseData = [];
        if (
          testCase.name[0] ===
          "An uncaught error was detected outside of a test"
        ) {
          testcaseData.push(data[index].file[0]);
        } else {
          if (testIndex === 0) {
            testcaseData.push(data[index - 1].file[0]);
          } else {
            testcaseData.push("");
          }
        }
        testcaseData.push(testCase.classname[0]);
        testcaseData.push(testCase.failure ? "Failed" : "Passed");
        if (testCase.failure) {
          testcaseData.push(testCase.failure[0].message);
        }
        return fileTestCaseData.push(testcaseData);
      });
    }
    return fileTestCaseData;
  });
};

const XMLToCSVWithFilter = (filePath) => {
  fs.readFile(filePath, (_err, data) => {
    parseString(data, { mergeAttrs: true }, (_err, results) => {
      const csvData = filterByFileAndTestCase(results.testsuites.testsuite);
      const removeEmptyRows = csvData.filter((item) => item.length > 0);
      const csvTitleAdded = [
        ["File Name", "Test Case", "Status", "Failure Message"],
        ...removeEmptyRows.flat(),
      ];
      const csv = arrayToCsv(csvTitleAdded);
      if (fs.existsSync("./mastery_report.csv")) {
        fs.unlinkSync("./mastery_report.csv");
      }
      fs.writeFile("mastery_report.csv", csv, (err) => {
        if (err) {
          console.log(err);
        }
      });
    });
  });
};

export default XMLToCSVWithFilter;
